import os
import ujson
import aicube
from media.camera import *
from media.display import *
from media.media import *
from time import *
import nncase_runtime as nn
import ulab.numpy as np
import time
import image
import random
import gc
import sys

DISPLAY_WIDTH = ALIGN_UP(1920, 16)
DISPLAY_HEIGHT = 1080

OUT_RGB888P_WIDTH = ALIGN_UP(640, 16)
OUT_RGB888P_HEIGH = 640

root_path="/sdcard/mp_deployment_source/"   # root_path要以/结尾
config_path= root_path + "deploy_config.json"
deploy_conf={}
debug_mode=1

class ScopedTiming:
    def __init__(self, info="", enable_profile=True):
        self.info = info
        self.enable_profile = enable_profile

    def __enter__(self):
        if self.enable_profile:
            self.start_time = time.time_ns()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        if self.enable_profile:
            elapsed_time = time.time_ns() - self.start_time
            print(f"{self.info} took {elapsed_time / 1000000:.2f} ms")

def read_deploy_config(config_path):
    # 打开JSON文件以进行读取deploy_config
    with open(config_path, 'r') as json_file:
        try:
            # 从文件中加载JSON数据
            config = ujson.load(json_file)

            # 打印数据（可根据需要执行其他操作）
            #print(config)
        except ValueError as e:
            print("JSON 解析错误:", e)
    return config

# 任务后处理

def get_pad_one_side_param(img_size):
    # 右padding或下padding
    dst_w = img_size[0]
    dst_h = img_size[1]

    ratio_w = dst_w / OUT_RGB888P_WIDTH
    ratio_h = dst_h / OUT_RGB888P_HEIGH
    if ratio_w < ratio_h:
        ratio = ratio_w
    else:
        ratio = ratio_h

    new_w = (int)(ratio * OUT_RGB888P_WIDTH)
    new_h = (int)(ratio * OUT_RGB888P_HEIGH)
    dw = (dst_w - new_w) / 2
    dh = (dst_h - new_h) / 2

    top = (int)(round(0))
    bottom = (int)(round(dh * 2 + 0.1))
    left = (int)(round(0))
    right = (int)(round(dw * 2 - 0.1))
    return [0, 0, 0, 0, top, bottom, left, right]

def ocr_detection():
    print("start")

    # 使用json读取内容初始化部署变量
    deploy_conf=read_deploy_config(config_path)
    kmodel_name=deploy_conf["kmodel_path"]
    mask_threshold=deploy_conf["mask_threshold"]
    box_threshold = deploy_conf["box_threshold"]
    img_size=deploy_conf["img_size"]
    # init kpu and load kmodel
    kpu = nn.kpu()
    ai2d = nn.ai2d()
    kpu.load_kmodel(root_path+kmodel_name)
    ai2d.set_dtype(nn.ai2d_format.NCHW_FMT,
                   nn.ai2d_format.NCHW_FMT,
                   np.uint8, np.uint8)
    ai2d.set_pad_param(True, get_pad_one_side_param(img_size), 0, [0, 0, 0])
    ai2d.set_resize_param(True, nn.interp_method.tf_bilinear, nn.interp_mode.half_pixel)
    ai2d_builder = ai2d.build([1,3,OUT_RGB888P_HEIGH,OUT_RGB888P_WIDTH], [1,3,img_size[0],img_size[1]])
    # use hdmi for display
    display.init(LT9611_1920X1080_30FPS)

    config = k_vb_config()
    config.max_pool_cnt = 1
    config.comm_pool[0].blk_size = 4*DISPLAY_WIDTH*DISPLAY_HEIGHT
    config.comm_pool[0].blk_cnt = 1
    config.comm_pool[0].mode = VB_REMAP_MODE_NOCACHE
    media.buffer_config(config)

    camera.sensor_init(CAM_DEV_ID_0, CAM_DEFAULT_SENSOR)

    # set chn0 output yuv420sp
    camera.set_outsize(CAM_DEV_ID_0, CAM_CHN_ID_0, DISPLAY_WIDTH, DISPLAY_HEIGHT)
    camera.set_outfmt(CAM_DEV_ID_0, CAM_CHN_ID_0, PIXEL_FORMAT_YUV_SEMIPLANAR_420)

    meida_source = media_device(CAMERA_MOD_ID, CAM_DEV_ID_0, CAM_CHN_ID_0)
    meida_sink = media_device(DISPLAY_MOD_ID, DISPLAY_DEV_ID, DISPLAY_CHN_VIDEO1)
    media.create_link(meida_source, meida_sink)

    display.set_plane(0, 0, DISPLAY_WIDTH, DISPLAY_HEIGHT, PIXEL_FORMAT_YVU_PLANAR_420, DISPLAY_MIRROR_NONE, DISPLAY_CHN_VIDEO1)

    # set chn2 output rgb88planar
    camera.set_outsize(CAM_DEV_ID_0, CAM_CHN_ID_2, OUT_RGB888P_WIDTH, OUT_RGB888P_HEIGH)
    camera.set_outfmt(CAM_DEV_ID_0, CAM_CHN_ID_2, PIXEL_FORMAT_RGB_888_PLANAR)

    try:
        media.buffer_init()

        buffer = media.request_buffer(4*DISPLAY_WIDTH*DISPLAY_HEIGHT)
        draw_img = image.Image(DISPLAY_WIDTH, DISPLAY_HEIGHT, image.ARGB8888)
        osd_img = image.Image(DISPLAY_WIDTH, DISPLAY_HEIGHT, image.ARGB8888, poolid=buffer.pool_id, alloc=image.ALLOC_VB, phyaddr=buffer.phys_addr, virtaddr=buffer.virt_addr)
        camera.start_stream(CAM_DEV_ID_0)

        ai2d_input_tensor = None
        data = np.ones((1,3,img_size[0],img_size[1]), dtype=np.uint8)
        ai2d_output_tensor = nn.from_numpy(data)
        count = 0
        while  True:
            os.exitpoint()
            with ScopedTiming("total",debug_mode > 0):
                rgb888p_img = camera.capture_image(CAM_DEV_ID_0, CAM_CHN_ID_2)

                # for rgb888planar
                if rgb888p_img.format() == image.RGBP888:
                    ai2d_input = rgb888p_img.to_numpy_ref()
                    ai2d_input_tensor = nn.from_numpy(ai2d_input)
                    ai2d_builder.run(ai2d_input_tensor, ai2d_output_tensor)
                    kpu.set_input_tensor(0, ai2d_output_tensor)
                    kpu.run()
                    results = []
                    for i in range(kpu.outputs_size()):
                        out_data = kpu.get_output_tensor(i)
                        result = out_data.to_numpy()
                        del out_data
                        results.append(result)
                    tmp = (ai2d_input.shape[0], ai2d_input.shape[1], ai2d_input.shape[2])
                    ai2d_input = ai2d_input.reshape((ai2d_input.shape[0], ai2d_input.shape[1] * ai2d_input.shape[2]))
                    ai2d_input = ai2d_input.transpose()
                    tmp2 = ai2d_input.copy()
                    tmp2 = tmp2.reshape((tmp[1], tmp[2], tmp[0]))
                    output_data = results[0][:, :, :, 0]
                    mp_list =  aicube.ocr_post_process(output_data.reshape(-1), tmp2.reshape(-1),
                                                        [img_size[0],img_size[1]],
                                                        [OUT_RGB888P_WIDTH, OUT_RGB888P_HEIGH], mask_threshold, box_threshold)
                    if mp_list:
                        draw_img.clear()
                        for j in mp_list:
                            for i in range(4):
                                x1 = j[1][(i * 2)]/OUT_RGB888P_WIDTH*DISPLAY_WIDTH
                                y1 = j[1][(i * 2 + 1)]/OUT_RGB888P_HEIGH*DISPLAY_HEIGHT
                                x2 = j[1][((i + 1) * 2) % 8]/OUT_RGB888P_WIDTH*DISPLAY_WIDTH
                                y2 = j[1][((i + 1) * 2 + 1) % 8]/OUT_RGB888P_HEIGH*DISPLAY_HEIGHT
                                draw_img.draw_line((int(x1), int(y1), int(x2), int(y2)), color=(255, 0, 0, 255),thickness=5)
                            draw_img.copy_to(osd_img)
                            display.show_image(osd_img, 0, 0, DISPLAY_CHN_OSD3)
                    else:
                        draw_img.clear()
                        draw_img.copy_to(osd_img)
                        display.show_image(osd_img, 0, 0, DISPLAY_CHN_OSD3)

                camera.release_image(CAM_DEV_ID_0, CAM_CHN_ID_2, rgb888p_img)

                gc.collect()

    except KeyboardInterrupt as e:
        print("user stop: ", e)
    except BaseException as e:
        sys.print_exception(e)
    finally:
        camera.stop_stream(CAM_DEV_ID_0)
        display.deinit()

        del ai2d
        del ai2d_builder
        if ai2d_input_tensor:
            del ai2d_input_tensor
        if ai2d_output_tensor:
            del ai2d_output_tensor
        del kpu
        gc.collect()
        nn.shrink_memory_pool()
        os.exitpoint(os.EXITPOINT_ENABLE_SLEEP)
        time.sleep_ms(100)
        media.release_buffer(buffer)
        media.destroy_link(meida_source, meida_sink)
        media.buffer_deinit()
    print("end")
    return 0


if __name__=="__main__":
    os.exitpoint(os.EXITPOINT_ENABLE)
    nn.shrink_memory_pool()
    ocr_detection()
